/* ----------------------------------------------------------------------------
 *         PalmSens EmStat SDK
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016, PalmSens BV
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * PalmSens's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY PALMSENS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL PALMSENS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */
 
/**
 * This is an example of how to use the EmStat SDK to connect your Arduino board to an EmStat.
 * The example allows the user to start measurements on the EmStat from the PC connected to the Arduino through USB.
 * 
 * Environment setup:
 * To run this example, you must include the EmStat SDK as a library first.
 * To do this, follow the menu "Sketch -> Include Library -> Add .ZIP Library..." and select the PalmSensComm folder.
 * You should now be able to compile the example.
 * 
 * Hardware setup:
 * To run this example, connect your Arduino "Serial1" port Rx, Tx and GND to the EmStat Serial Tx, Rx and GND respectively.
 * See the documentation of your Arduino board and the document "EmStat## for OEM.pdf" for the EmStat to find these pins.
 * Note:  This example assumes that "Serial" is your PC connection (probably USB) and the EmStat is connected on "Serial1".
 *        If your board does not have a "Serial1", read the paragraph at the bottom of this guide.
 * The Arduino board should be connected normally to a PC. If not powering the EmStat by other means, the EmStat should 
 * be connected to the PC through USB for power.
 * 
 * How to use:
 * First change the device type you are using. Do this in the variable called "Device". Then, compile and upload this sketch
 * through the Arduino IDE. Next, open a serial monitor to the Arduino (you can do this from the Arduino IDE). 
 * You should see messages being printed containing measured data from the EmStat. 
 * You can now send commands to the Arduino, possible commands are:
 * 
 * s: Toggles the printing of all sent messages to the EmStat.
 * r: Toggles the printing of all received messages from the EmStat.
 * v: Prints the version of the connected EmStat.
 * p: Manually set the output potential to 0.5 V.
 * c: Set cell ON
 * m: Set multiplexer channel / digital IO.
 * 1 to 9: Start a pre-set measurement.
 * 
 * Important: 
 * Some Arduino boards only have one hardware serial port (Arduino Uno, for example), which is usually used by USB. 
 * So if we want to communicate with the PC through USB (for programming and debugging) and an EmStat at the same time, 
 * we need to use a SoftwareSerial port for the EmStat. 
 * However, the SoftwareSerial cannot always reach the default communication speed of the EmStat (230400 Baud), 
 * so the SoftwareSerial is set to 57600 Baud instead. You need to change the EmStat's Baud rate to 57600 yourself.
 * This can be done in PSTrace: Tools -> Instrument Settings -> Set Baud Rate.
 * Once this is done you can connect the EmStat to your Arduino via pin 10 (Rx), pin 11 (Tx) and GND on your Arduino.
 * If you are using a board with at least 2 hardware serial ports (Serial and Serial1) you don't need to do anything.
 * 
 * Author(s): Hielke Veringa (hielke@palmsens.com)
 */

//Because PalmSensComm is a C library and Arduino uses a C++ compiler, we must use the "extern "C"" wrapper.
extern "C" {
  #include <PSComm.h>
};

//The following ifndef statement is not really important for the example, it just ensures there's always a Serial1 port.
//It also sets the communication speed to the EmStat to 57600 Baud if using a SoftwareSerial port.
#ifndef PIN_SERIAL1_RX
  #include <SoftwareSerial.h>
  //Probably only has one hardware serial port, so we have to use SoftwareSerial and are limited to 57600 Baud.
  #define EMSTAT_BAUDRATE 57600
  SoftwareSerial softwareSerial(6, 7); // RX pin, TX pin
  #define Serial1 softwareSerial
#else
  //Has at least 2 hardware serial ports, can use full speed
  #define EMSTAT_BAUDRATE 230400
#endif

//Set "Device" to the device you are using.
//Possible devices are:   EMSTAT2, EMSTAT3 or EMSTAT3P.
const DeviceType Device = EMSTAT3;

bool PrintSent = false;
bool PrintReceived = false;

//This contains the parameters for a measurement
Method m;
//The EmStat communication object.
PSComm psc;

//We have to give PSComm some functions to communicate with the EmStat (in PSCommInit).
//However, because the C compiler doesn't understand C++ classes,
//we must wrap the write/read functions from the Serial class in a normal function, first.
//We are using Serial and Serial1 here, but you can use any serial port.
int write_wrapper(char c)
{
  if(PrintSent == true)
  {
    //Send all data to PC as well for debugging purposes
    Serial.write(c);
  }
  return Serial1.write(c);
}

int read_wrapper()
{
  int c = Serial1.read();
  
  if(PrintReceived == true && c != -1) //-1 means no data
  {
    //Send all received data to PC for debugging purposes
    Serial.write(c);
  }
  return c;
}

void setup()
{
  // put your setup code here, to run once:
  
  //Init serial ports
  Serial.begin(230400);
  Serial1.begin(EMSTAT_BAUDRATE);

  //Init PSComm struct (one for every EmStat).
  PSCommInit(&psc, Device, 2000, &write_wrapper, &read_wrapper, &millis);
}

void loop() 
{
  // put your main code here, to run repeatedly:

  MeasureData data;
  //If we have any buffered messages waiting for us
  while (Serial1.available())
  {
    //Try to identify and parse a package
    RetCode code = ReceivePackage(&psc, &m, &data);
    if(code == CODE_OK)
    {
      //Received valid package, print it.
      Serial.print("Potential: ");
      Serial.print(data.potential, 3);
      Serial.print(" Current: ");
      Serial.print(CurrentToCR(data.current, data.crange), 3);
      Serial.print(" in range ");
      Serial.print(GetCRString(data.crange));
      Serial.print(" aux_input: ");
      Serial.print(data.aux_input, 3);
      Serial.print(" stage: ");
      Serial.print(data.phase);
      Serial.print(" status: ");
      Serial.print(data.status);
      Serial.print(" noise: ");
      Serial.println(data.noise, 3);
    }
    else if(code == CODE_MEASUREMENT_DONE)
    {
      Serial.println("Measurement finished.");
    }
    else if(code == CODE_MEASURING)
    {
      Serial.println("Pretreatment done, starting measurement.");
    }
    else
    {
      //Failed to parse or identify package.
      Serial.print("Failed to parse package: ");
      Serial.println(code);
    }
  }

  //The rest of the code in this function is just so you can give the Arduino commands from the PC.
  if(Serial.available())
  {
    char cmd2 = Serial.read();
    switch (cmd2)
    {
      case 's':
        //Toggle DebugSent
        PrintSent = !PrintSent;
        break;
      case 'r':
        //Toggle DebugReceived
        PrintReceived = !PrintReceived;
        break;
      case 'v':
        //First allocate space for our version string
        char version_str[VERSION_STR_LENGTH];
        if (QueryVersion(&psc, version_str, VERSION_STR_LENGTH) != CODE_OK)
        {
          Serial.write("Cannot query version.\r\n");
        }
        else
        {
          Serial.write("Hello, ");
          Serial.write(version_str);
          Serial.write("\r\n");
        }
        break;
      case 'p':
        if (MCSetPotential(&psc, 0.500) != CODE_OK)
        {
          Serial.write("Cannot set potential.\r\n");
        }
        break;
      case 'c':
        if (MCSetCell(&psc, CR_1uA, CELL_ON) != CODE_OK)
        {
          Serial.write("Cannot set cell on.\r\n");
        }
        break;
      case 'm':
        //Set digital out / multiplexer channel
        if (MCSetDigital(&psc, Serial.read()) != CODE_OK)
        {
          Serial.write("Cannot set channel.\r\n");
        }
        break;
      case '1':
        SetLinearSweepParams(&m, psc.device,
          0, 4.0, 1.5, 4.0, 
          6.0, CR_10nA, CR_1mA, 
          -0.5, 0.5, 0.005, 0.1, 
          OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Linear Sweep measurement started.\r\n");
        }
        break;
      case '2':
        SetDifferentialPulseParams(&m, psc.device,
          0, 0.0, 1.5, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          -1.0, 1.0, 0.005, 0.01, 0.01, 
          0.25, OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Differential Pulse measurement started.\r\n");
        }
        break;
      case '3':
        SetSquareWaveParams(&m, psc.device,
          0.0, 0.0, 0.0, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          -1.0, 1.0, 0.01, 0.01, 100.0, 
          OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Square Wave measurement started.\r\n");
        }
        break;
      case '4':
        SetNormalPulseParams(&m, psc.device,
          0.1, 0.0, 0.2, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          -1.0, 1.0, 0.01, 0.05, 0.1, 
          OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Normal Pulse measurement started.\r\n");
        }
        break;
      case '5':
        SetCyclicVoltammetryParams(&m, psc.device,
          0.1, 0.0, 0.2, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          0.0, -1.0, 1.0, 0.1, 0.5, 2,
          OPTIONS_NONE, 0.0, 50);

        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Cyclic Voltammetry measurement started.\r\n");
        }
        break;
      case '6':
        SetAmperometricDetectionParams(&m, psc.device,
          0.1, 0.0, 0.2, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          1.0, 0.01, 10.0,
          OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Amperometric Detection measurement started.\r\n");
        }
        break;
      case '7':
        SetPulsedAmperometricDetectionParams(&m, psc.device,
          0.1, 0.0, 0.2, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          1.0, 0.2, 0.02, PAD_MODES_DC, 0.1, 10.0,
          OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Pulsed Amperometric Detection measurement started.\r\n");
        }
        break;
      case '8':
        SetOpenCircuitPotentiometryParams(&m, psc.device,
          0.1, 0.0, 0.2, 0.0, 
          0.1, 10.0,
          OPTIONS_NONE, 0.0, 50);   
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Open Circuit Potentiometry measurement started.\r\n");
        }
        break;
      case '9':
        SetMultiplePulseAmperometryParams(&m, psc.device,
          0.1, 0.0, 0.2, 0.0, 
          1.0, CR_10nA, CR_1mA, 
          0.5, 0.1, -0.5, 0.1, 1.0, 0.2, 10.0,
          OPTIONS_NONE, 0.0, 50);
          
        if (StartMeasurement(&psc, &m) != CODE_OK )
        {
          Serial.write("Cannot start measurement.\r\n");
        }
        else
        {
          Serial.write("Multiple Pulse Amperometry measurement started.\r\n");
        }
        break;    
    }
  }
}
